jQuery.sap.declare("sas.stress.wkbench.widget.CategoryFilter");

jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

sas.hc.m.Page.extend("sas.stress.wkbench.widget.CategoryFilter", {

    _list: null,
    _title: null,
    _toolbar: null,

    metadata: {
        properties: {
            actionButtons: { type: "any[]", defaultValue: null, bindable: true },
            itemsPath: { type: "string", defaultValue: null, bindable: true },
            itemTemplate: { type: "any", defaultValue: null, bindable: true },
            selectedCategory: { type: "string", defaultValue: null, bindable: true },
            selectedItem: { type: "any", defaultValue: null, bindable: true }
        },
        publicMethods: ["getCategoriesList", "getListItems"],
        aggregations: {
            "list": { type: "sap.ui.core.Control", multiple: false, visibility: "hidden" }
        },
        events: {
            "selectionChange": {}
        }
    },

    getCategoriesList: function () {
        return this._list;
    },

    getListItems: function () {
        if (this._list) {
            return this._list.getItems();
        }
    },

    setTitle: function (titleText) {
        if (this._title) {
            this._title.setText(titleText);
        }
        if (this._list) {
            this._list.setAriaLabel(titleText);
        }
    },

     setActionButtons: function (actionButtons) {
         if (this._toolbar && actionButtons.length > 0) {
             var toolbar = this._toolbar;
             _.each(actionButtons, function (button) {
                 toolbar.addContent(button);
             });
         }
     },

    setSelectedCategory: function (category, fireEvent) {
        this.setProperty("selectedCategory", category, true);
        if (this._list && this._list.getItems().length > 0) {
            if (!!category) {
                var selectedItem = _.find(this._list.getItems(), function (item) {
                    return (item.getTitle() === category);
                } );
            }
            if (!selectedItem) {
                selectedItem = this._list.getItems()[0];
            }
            this.setSelectedItem(selectedItem, fireEvent);
        }
        return this;
    },

    setSelectedItem: function (item, fireEvent) {
        this.setProperty("selectedItem", item, true);
        if (this._list) {
            this._list.removeSelections(true);
            //var itemToSelectName = item.name||item.getProperty("title");
            var listItem=_.find(this._list.getItems(), function(itm){
                return itm.getProperty("title")===(item.name||item.getProperty("title"));
            });
            this._list.setSelectedItem(listItem, true, fireEvent);
        }
        return this;
    },

    renderer: {},

    rb: sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench")
});

sas.stress.wkbench.widget.CategoryFilter.DEFAULT_WIDTH = "260px";

sas.stress.wkbench.widget.CategoryFilter.prototype.init = function () {
    if (sas.hc.m.Page.prototype.init) {   // check whether superclass has an init() method
        sas.hc.m.Page.prototype.init.apply(this, arguments);  // call super.init()
    }

    this._toolbar = this._createToolbar();
    this.setCustomHeader(this._toolbar);

    this._list = this._createCategoriesList();
    this.addContent(this._list);
};

sas.stress.wkbench.widget.CategoryFilter.prototype.applySettings = function () {
    if (sas.hc.m.Page.prototype.applySettings) {
        sas.hc.m.Page.prototype.applySettings.apply(this, arguments);
    }
    if (this._list) {
        this._list.bindAggregation("items", this.getItemsPath(), this.getItemTemplate());
        var selectedCategory = this.getSelectedCategory();
        if (selectedCategory) {
            this.setSelectedCategory(selectedCategory, true);
        }
    }
};

sas.stress.wkbench.widget.CategoryFilter.prototype.onBeforeRendering = function () {
    if (sas.hc.m.Page.prototype.onBeforeRendering) {
        sas.hc.m.Page.prototype.onBeforeRendering.apply(this, arguments);
    }
};

sas.stress.wkbench.widget.CategoryFilter.prototype._createToolbar = function () {
    this._title = new sas.hc.m.Label({
        text: this.rb.getText("categoryFilter.header.title")
    });
    return new sas.hc.m.ResponsiveToolbar({
        content: [
            this._title,
            new sas.hc.m.ToolbarSpacer()
        ]
    });
};

sas.stress.wkbench.widget.CategoryFilter.prototype._createCategoriesList = function () {
    return new sas.hc.m.List({
        ariaLabel: this.rb.getText("categoryFilter.header.title"),
        selectionChange: [this._onSelectionChange, this],
        height: "100%"
    });
};

sas.stress.wkbench.widget.CategoryFilter.prototype._onSelectionChange = function (event) {
    var selectedItem = this._list.getSelectedItem();
    this.setProperty("selectedItem", selectedItem, true);
    this.setProperty("selectedCategory", selectedItem.getTitle(), true);

    this.fireSelectionChange({ sourceEvent: event });
};
